% download training data from http://yann.lecun.com/exdb/mnist/
% and the corresponding matlab helpers to read the data from
% http://ufldl.stanford.edu/wiki/index.php/Using_the_MNIST_Dataset

clear all;
close all;

rng(1)

images = loadMNISTImages('t10k-images-idx3-ubyte');
labels = loadMNISTLabels('t10k-labels-idx1-ubyte')+1;
test_data = [images; labels'];

images = loadMNISTImages('train-images-idx3-ubyte');
[dim, N] = size(images);
labels = loadMNISTLabels('train-labels-idx1-ubyte')+1;
C=max(labels);
train_data = [images; labels'];
X = train_data(1:dim, :)';
%Normalize data
%X = X/max(X(:));

y = train_data(end, :)';

%%
lambda1 = 0.001;
lambda2 = 0.002;

% compute step size.
L = norm(X,'fro')^2/N + lambda1;
%tau = 1/L;
tau = 0.7;


max_iter=2000;



% compute classifier scores
classifier = @(W, b, X) X*W + repmat(b, size(X, 1), 1);

% evaluate loss given classifier scores
scores = @(classifier) exp(classifier) ./ repmat(sum(exp(classifier), 2), 1, C);
loss = @(scores) sum(-log(scores(sub2ind(size(scores), (1:N)', y))), 1);

objective = @(W, b) loss(scores(classifier(W, b, X)))/N ...
    + 0.5 * lambda1 * sum(W(:).^2)...
    + 0.5 * lambda1 * sum(b(:).^2)...
    + lambda2 * sum(W(:));

% compute gradient
grad_W = @(W, scores, j) ...
    sum(X.*repmat(scores(:, j) - double(y == j), 1, dim), 1)'/N  ...
    + lambda1 * W(:, j);

grad_b = @(b, scores, j) ...
    sum(scores(:, j) - double(y == j), 1)'/N  ...
    + lambda1 * b(j);


% initialize variables
W = rand(dim, C);
b = zeros(1, C);


obj_val = zeros(1, max_iter);
test_error = zeros(1, max_iter);
train_error = zeros(1, max_iter);

% Do derivative check (for debugging only).
% objective_diff = @(W, b) loss(scores(classifier(W, b, X)))/N ...
%    + 0.5 * lambda1 * sum(W(:).^2)...
%    + 0.5 * lambda1 * sum(b(:).^2);
% func = @(x) fun(x, dim, C, X, scores, classifier, objective_diff, grad_W, grad_b);
% 
% options = optimoptions('fminunc','GradObj','on','DerivativeCheck','on');
% res = fminunc(func, [W(:);b(:)], options);
% W = reshape(res(1:dim*C), dim, C);
% b = reshape(res(dim*C+1:end), 1, C);


% apply proximal gradient descent scheme
for iter=1:max_iter
    scores_train = scores(classifier(W, b, X));
    scores_test = scores(classifier(W, b, test_data(1:dim,:)'));
    
    [~, predicted_classes_train] = max(scores_train');
    [~, predicted_classes_test] = max(scores_test');
    
    % compute current objective value, test error and training error
    obj_val(iter) = objective(W, b);
    train_error(iter) = nnz(predicted_classes_train'~=train_data(end,:)')/size(train_data,2)*100;
    test_error(iter) = nnz(predicted_classes_test'~=test_data(end,:)')/size(test_data,2)*100;
  
    % print stuff
    fprintf('It=%d   obj=%f   train_error=%2.2f%%   test_error=%2.2f%%\n', ...
        iter, obj_val(iter), train_error(iter), test_error(iter));
    
    % do a gradient descent step on the differentiable part of the
    % objective.
    for j=1:C   
        W(:, j) = W(:, j) - tau*grad_W(W, scores_train, j);
        b(j) = b(j) - tau*grad_b(b, scores_train, j);
    end
    
    % Apply proximal operator of 1-norm to W^t:
    % Apply shrinkage to the current iterate W^t
    W = max(abs(W)-tau*lambda2, 0).*sign(W);

    
    % evalualte stopping criterion
    if(iter > 1 && obj_val(iter-1) - obj_val(iter) < 1e-12)
        break;
    end
end

%% Plot objective value and test error
figure;
subplot(1,3,1);
plot(obj_val(1:iter));
title('Objective value');
subplot(1,3,2);
plot(train_error(1:iter));
title('Train error');
subplot(1,3,3);
plot(test_error(1:iter));
title('Test error');

